from abc import ABC, abstractmethod
from typing import Callable, Union, Literal

from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.wait import WebDriverWait, D, T

import drivers
import utils
from models.collect_data import CollectData
from stores.data_store_interface import IDataStore


class IDataCollectionAdapter(ABC):
    """
    数据收集适配器抽象类
    """

    _url = ""
    _store = None
    _driver = None
    _adapter_type = ""
    _cur_keyword = None
    _keywords = None
    _keyword_array = None
    _error_count = 0
    _max_error_count = utils.get_config_int("adapter.max_error_count", 3)

    _err_keywords = {}

    @property
    def search_day_key(self) -> str:
        return f"adapter.{self._adapter_type}.search_day"

    @property
    def batch_save_key(self) -> str:
        return f"adapter.{self._adapter_type}.batch_save"

    @property
    def store(self) -> IDataStore:
        return self._store

    @property
    def url(self):
        return self._url

    @property
    def cur_keyword(self):
        return self._cur_keyword

    @property
    def keywords(self):
        return self._keywords

    @property
    def keyword_array(self):
        return self._keyword_array

    @property
    def driver(self) -> webdriver:
        if not self._driver:
            try:
                self._driver = drivers.gen_driver(self.url)
            except Exception as e:
                raise Exception(f"创建驱动器失败: {e}")
        return self._driver

    def collect(self, keywords: str, store: IDataStore) -> None:
        """
        处理搜索结果列表，返回处理后的数据列表

        :param keywords: 搜索结果列表
        :param store: 数据储存库
        :type keywords: str
        :return: 处理后的数据列表
        :rtype: list
        :raises Exception: 如果处理失败，应抛出异常
        """
        if store:
            self._store = store
        if not keywords:
            raise Exception("未指定搜索关键字")
        utils.get_logger().info(f"开始采集: {keywords}")
        self._error_count = 0
        self._keyword_array = keywords.split(",")
        count = 0
        for keyword in self._keyword_array:
            if not keyword:
                continue
            try:
                count += 1
                self._cur_keyword = keyword
                utils.get_logger().info(f"采集关键字[{count}]: {keyword}")
                self._error_count = 0
                self._collect(keyword)
                if self.cur_keyword in self._err_keywords:
                    del self._err_keywords[self.cur_keyword]  # 删除键
            except Exception as e:
                utils.get_logger().error(f"==> {e}")
            # except Exception as e:
            #     raise Exception(f"采集数据失败: {e}")
        self._collect_error_keywors()

    @abstractmethod
    def login(self, username: str, password: str) -> None:
        """
        如果需要登录，则登录后跳转到搜索页面（不自动跳转的需要手动执行）

        :param username: 用户名
        :type username: str
        :param password: 密码
        :type password: str
        :raises Exception: 如果登录失败，应抛出异常
        """
        try:
            # 实现登录逻辑
            pass
        except Exception as e:
            raise Exception(f"登录失败: {e}")

    def _wait(self, timeout=20, poll_frequency=1):
        return WebDriverWait(self.driver, timeout, poll_frequency)

    def _wait_until(
        self,
        method: Callable[[D], Union[Literal[False], T]],
        timeout=20,
        poll_frequency=1,
    ) -> bool:
        try:
            self._wait(timeout, poll_frequency).until(method)
            return True
        except TimeoutException:
            err_count = (
                self._err_keywords[self.cur_keyword]
                if self.cur_keyword in self._err_keywords
                else 0
            )
            err_count += 1
            utils.get_logger().error(
                f"采集数据 超时 [{self.cur_keyword}][{err_count}/{self._max_error_count}]"
            )
            self._err_keywords[self.cur_keyword] = err_count
            if err_count > self._max_error_count:
                del self._err_keywords[self.cur_keyword]  # 删除键
            return False
            # raise TimeoutException(
            #     f"采集数据 超时 {self.cur_keyword} [{err_count}/{self._max_error_count}]"
            # )

    def _collect_error_keywors(self):
        if not self._err_keywords:
            return
        for keyword, err_count in self._err_keywords.items():
            try:
                utils.get_logger().info(
                    f"重新采集错误关键字[{err_count}/{self._max_error_count}]: {keyword}"
                )
                self._cur_keyword = keyword
                self._collect(keyword)
                if self.cur_keyword in self._err_keywords:
                    del self._err_keywords[self.cur_keyword]  # 删除键
            except Exception as e:
                utils.get_logger().error(f"失败: {e}")
        self._collect_error_keywors()

    @abstractmethod
    def _collect(self, keyword: str) -> None:
        """
        根据关键字采集
        :param keyword: 搜索关键字
        :type keyword: str
        """
        pass

    def teardown(self) -> None:
        """
        关闭浏览器驱动器

        :raises Exception: 如果关闭驱动器失败，应抛出异常
        """
        try:
            if self.driver:
                self.driver.quit()
        except Exception as e:
            raise Exception(f"关闭驱动器失败: {e}")

    def _check_is_collect_by_url(self, url: str) -> bool:
        old = self.store.query_one_collect_url(url)
        if old:
            utils.get_logger().debug(f"已采集过: {url}")
            return True
        return False

    def _check_content(self, content) -> bool:
        collect_data_key = utils.get_config_value("save.collect_data_key")
        if not collect_data_key:
            utils.get_logger().info("未配置 save.collect_data_key，跳过内容检查")
            return True
        # utils.get_logger().info(f"检查数据有效性: {collect_data_key}")
        collect_data_key = collect_data_key.replace("，", ",")
        keys = collect_data_key.split(",")
        keys = [key.strip() for key in keys]
        for key in keys:
            key = key.strip()
            # utils.get_logger().info(f"检查数据有效性: {key}")
            if key in content:
                utils.get_logger().info(f"有效数据: {self.driver.current_url}")
                return True

        return False

    def _save_db(self, url, content, data_type=0, attach_str=None, is_invalid=False):
        if not self.store:
            utils.get_logger().info(
                f"DataStore 未指定: {url}，关键字{self.cur_keyword}"
            )
            return False
        else:
            status = 2 if is_invalid else 0
            data = CollectData(
                url=url,
                keyword=self.cur_keyword,
                content=content,
                data_type=data_type,
                attach_path=attach_str,
                status=status,
            )
            self.store.insert_collect_data(
                data, utils.get_config_bool(self.batch_save_key)
            )
            return True
